public class UndoMap<K,V> extends Map<K,V> {
    private K prevKey;
    private V prevVal;

    public UndoMap() {
        super();
        prevKey = null;
        prevVal = null;
    }

    public void put(K k, V v) {
        prevKey = k;
        if (super.isRegistered(k)) {
            prevVal = super.get(k);
        } else {
            prevVal = null;
        }
        super.put(k,v);
    }

    public void undo() {
        if (prevKey != null) {
            if (prevVal != null) {
                super.put(prevKey,prevVal);
                prevVal = null;
            } else {
                super.delete(prevKey);
            }
            prevKey = null;
        }
    }
}